#include "nix_compat.h"

inline std::string& strlwr(std::string& s) {
  std::transform(s.begin(), s.end(), s.begin(), std::ptr_fun<int, int>(std::tolower));
  return s;
}

inline std::string strlower(const std::string& s) {
  std::string result(s);
  return strlwr(result);
}

void _strlwr_s(char* str, size_t sizeInBytes) {
    std::string ls = strlower(std::string(str));
    for(int i = 0; i < sizeInBytes; ++i) {
        str[i] = ls[i];
    }
    strcpy(str, ls.c_str());
}

void _ltoa_s(long value, char* str, size_t sizeOfstr, int radix) {
    std::stringstream ss;
    ss << value;
    strcpy(str, ss.str().c_str());
}

void _ultoa_s(unsigned long value, char* str, size_t sizeOfstr, int radix) {
    _ltoa_s(value, str, sizeOfstr, radix);
}

void _gcvt_s(char* buffer, size_t sizeInBytes, double value, int digits) {
    std::stringstream ss;
    ss << value;
    strcpy(buffer, ss.str().c_str());
}

#ifdef __linux__

uint32_t GetTickCount() {
    struct timespec ts;
    clock_gettime(0, &ts);
    uint32_t ticks = ts.tv_nsec / 1e+06 + ts.tv_sec * 1e+03;
    return ticks;
}

#endif // __linux__

#ifdef __APPLE__

#include <mach/mach_time.h>
#define ORWL_NANO (+1.0E-9)
#define ORWL_GIGA UINT64_C(1000000000)

static double orwl_timebase = 0.0;
static uint64_t orwl_timestart = 0;

struct timespec orwl_gettime(void) {
  // be more careful in a multithreaded environement
  if (!orwl_timestart) {
    mach_timebase_info_data_t tb = { 0 };
    mach_timebase_info(&tb);
    orwl_timebase = tb.numer;
    orwl_timebase /= tb.denom;
    orwl_timestart = mach_absolute_time();
  }
  struct timespec t;
  double diff = (mach_absolute_time() - orwl_timestart) * orwl_timebase;
  t.tv_sec = diff * ORWL_NANO;
  t.tv_nsec = diff - (t.tv_sec * ORWL_GIGA);
  return t;
}

uint32_t GetTickCount() {
    struct timespec ts;
    orwl_gettime();
    uint32_t ticks = ts.tv_nsec / 1e+06 + ts.tv_sec * 1e+03;
    return ticks;
}

#endif // __APPLE__