#include <stdint.h>
#include <string>
#include "mex.h"


void mfpt_est_iterate(
		const uint64_t *wt_ir, const uint64_t *wt_jc, const double *wt_pr,
		uint64_t n, uint64_t nit,
		const uint64_t *idx, uint64_t nidx, const bool *is_in_idx,
		uint64_t nthreads,
		mxArray *&msum, mxArray *&mnum
);


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
	// nargs
	if (nrhs != 4)
		mexErrMsgTxt(("mfpt_est_iterate_mex: got " + std::to_string(nrhs) + " input arguments; 4 was expected.\n").c_str());
	if (nlhs != 2)
		mexErrMsgTxt(("mfpt_est_iterate_mex: got " + std::to_string(nlhs) + " output arguments; 2 was expected.\n").c_str());

	// W'
	mxArray *wt = (mxArray*)prhs[0];
	if (!mxIsSparse(wt))
		mexErrMsgTxt("mfpt_est_iterate_mex: state transition matrix transpose is not sparse.\n");
	uint64_t *wt_ir = (uint64_t*)mxGetIr(wt);
	uint64_t *wt_jc = (uint64_t*)mxGetJc(wt);
	double *wt_pr = (double*)mxGetPr(wt);
	
	// n
	uint64_t n = mxGetN(wt);
	if (mxGetM(wt) != n)
		mexErrMsgTxt("mfpt_est_iterate_mex: state transition matrix transpose is not square.\n");

	// nit
	uint64_t nit = (uint64_t)mxGetScalar(prhs[1]);

	// idx, nidx
	if (mxIsDouble(prhs[2]))
		mexErrMsgTxt("mfpt_est_iterate_mex: idx must be uint64. Use uint64(idx) if necessary.\n");
	uint64_t *idx = (uint64_t*)mxGetPr(prhs[2]);
	uint64_t nidx = mxGetNumberOfElements(prhs[2]);
	
	// nthreads
	if (mxIsDouble(prhs[3]))
		mexErrMsgTxt("mfpt_est_iterate_mex: nthreads must be integer.\n");
	uint64_t nthreads = (uint64_t)mxGetScalar(prhs[3]);

	// is_in_idx (+ idx validation)
	bool *is_in_idx = (bool*)mxCalloc(n, sizeof(bool));
	for (uint64_t ii = 0; ii < nidx; ++ii) {
		uint64_t i = idx[ii];
		is_in_idx[i] = true;
		if ((ii < nidx - 1) && (i >= idx[ii + 1]))
			mexErrMsgTxt(("mfpt_est_iterate_mex: idx are either unsorted or non-unique (" + std::to_string(i + 1) + " >= " + std::to_string(idx[ii + 1] + 1) + ").").c_str());
	}
	
	// iterating
	mxArray *msum, *mnum = NULL;
	
	mfpt_est_iterate(wt_ir, wt_jc, wt_pr, n, nit, idx, nidx, is_in_idx, nthreads, msum, mnum);
	
	plhs[0] = msum;
	plhs[1] = mnum;
}